@@ -1,19 +1,15 @@
 # -*- coding: utf-8 -*-
-try:
-    import pygtk
-    pygtk.require('2.0')
-except ImportError, err:
-    print ("PyGTK not found. Please make sure it is installed properly and referenced in your PYTHONPATH environment variable.")
-
 import os.path
 import threading
 
-import gobject
-import gtk
-import pango
+import gi
+gi.require_version("Gtk", "3.0")
+
+from gi.repository import Gtk,GdkPixbuf,GObject,Pango,Gdk
 from gettext import gettext as _
 from gettext import ngettext
 
+import cairo
 import dialog_swallower
 import game_selector
 import gsudoku
@@ -27,23 +23,6 @@ from defaults import (APPNAME, APPNAME_SHORT, AUTHORS, COPYRIGHT, DESCRIPTION, D
 from gtk_goodies import gconf_wrapper, Undo, dialog_extras
 from simple_debug import simple_debug, options
 
-ICON_FACTORY = gtk.IconFactory()
-STOCK_PIXBUFS = {}
-for filename, stock_id in [('footprints.png', 'tracks'), ]:
-    try:
-        pb = gtk.gdk.pixbuf_new_from_file(os.path.join(IMAGE_DIR, filename))
-    except gobject.GError, e:
-        print 'Failed to load pixbuf: %s' % e
-        continue
-    STOCK_PIXBUFS[stock_id] = pb
-    iconset = gtk.IconSet(pb)
-    ICON_FACTORY.add(stock_id, iconset)
-    ICON_FACTORY.add_default()
-
-gtk.stock_add([('tracks',
-                _('Track moves'),
-                0, 0, ""), ])
-
 def inactivate_new_game_etc (fun):
     def inactivate_new_game_etc_ (ui, *args, **kwargs):
         paths = [
@@ -152,7 +131,7 @@ class UI (gconf_wrapper.GConfWrapper):
         and allow a tester to set up a game programmatically.
         """
         gconf_wrapper.GConfWrapper.__init__(self,
-                                            gconf_wrapper.GConf('gnome-sudoku')
+                                            gconf_wrapper.GConfWrap('gnome-sudoku')
                                             )
         self.setup_gui()
         self.timer = timer.ActiveTimer(self.w)
@@ -176,7 +155,7 @@ class UI (gconf_wrapper.GConfWrapper):
                 self.quit = False
 
         # Generate puzzles in background...
-        gobject.timeout_add_seconds(1, lambda *args: self.start_worker_thread() and True)
+        GObject.timeout_add_seconds(1, lambda *args: self.start_worker_thread() and True)
 
     @inactivate_new_game_etc
     def select_game (self):
@@ -217,43 +196,43 @@ class UI (gconf_wrapper.GConfWrapper):
         self.setup_toggles()
 
     def setup_main_window (self):
-        gtk.window_set_default_icon_name('gnome-sudoku')
-        self.w = gtk.Window()
+        Gtk.Window.set_default_icon_name('gnome-sudoku')
+        self.w = Gtk.Window()
         self.w.set_default_size(self.gconf['width'], self.gconf['height'])
         self.w.set_title(APPNAME_SHORT)
         self.w.connect('configure-event', self.resize_cb)
         self.w.connect('delete-event', self.quit_cb)
-        self.uimanager = gtk.UIManager()
+        self.uimanager = Gtk.UIManager()
 
     def setup_actions (self):
-        self.main_actions = gtk.ActionGroup('MainActions')
+        self.main_actions = Gtk.ActionGroup('MainActions')
         self.main_actions.add_actions([
             ('Game', None, _('_Game')),
-            ('New', gtk.STOCK_NEW, None, '<Control>n', _('New game'), self.new_cb),
-            ('Reset', gtk.STOCK_CLEAR, _('_Reset'), '<Control>b',
+            ('New', Gtk.STOCK_NEW, None, '<Control>n', _('New game'), self.new_cb),
+            ('Reset', Gtk.STOCK_CLEAR, _('_Reset'), '<Control>b',
              None, self.game_reset_cb),
-            ('Undo', gtk.STOCK_UNDO, _('_Undo'), '<Control>z',
+            ('Undo', Gtk.STOCK_UNDO, _('_Undo'), '<Control>z',
              _('Undo last action'), self.stop_dancer),
-            ('Redo', gtk.STOCK_REDO, _('_Redo'), '<Shift><Control>z',
+            ('Redo', Gtk.STOCK_REDO, _('_Redo'), '<Shift><Control>z',
              _('Redo last action')),
-            ('PuzzleInfo', gtk.STOCK_ABOUT, _('Puzzle _Statistics...'), None,
+            ('PuzzleInfo', Gtk.STOCK_ABOUT, _('Puzzle _Statistics...'), None,
              None, self.show_info_cb),
-            ('Print', gtk.STOCK_PRINT, _('_Print...'), '<Control>p', None, self.print_game),
-            ('PrintMany', gtk.STOCK_PRINT, _('Print _Multiple Sudokus...'), None,
+            ('Print', Gtk.STOCK_PRINT, _('_Print...'), '<Control>p', None, self.print_game),
+            ('PrintMany', Gtk.STOCK_PRINT, _('Print _Multiple Sudokus...'), None,
              None, self.print_multiple_games),
-            ('Close', gtk.STOCK_CLOSE, None, '<Control>w', None, self.quit_cb),
+            ('Close', Gtk.STOCK_CLOSE, None, '<Control>w', None, self.quit_cb),
             ('Settings', None, _('_Settings')),
-            ('FullScreen', gtk.STOCK_FULLSCREEN, None, 'F11', None, self.full_screen_cb),
+            ('FullScreen', Gtk.STOCK_FULLSCREEN, None, 'F11', None, self.full_screen_cb),
             ('Tools', None, _('_Tools')),
-            ('ShowPossible', gtk.STOCK_DIALOG_INFO, _('_Hint'), '<Control>h',
+            ('ShowPossible', Gtk.STOCK_DIALOG_INFO, _('_Hint'), '<Control>h',
              _('Show a square that is easy to fill.'), self.show_hint_cb),
             ('ClearTopNotes', None, _('Clear _Top Notes'), '<Control>j',
              None, self.clear_top_notes_cb),
             ('ClearBottomNotes', None, _('Clear _Bottom Notes'), '<Control>k',
              None, self.clear_bottom_notes_cb),
             ('Help', None, _('_Help'), None, None, None),
-            ('ShowHelp', gtk.STOCK_HELP, _('_Contents'), 'F1', None, self.show_help),
-            ('About', gtk.STOCK_ABOUT, None, None, None, self.show_about),
+            ('ShowHelp', Gtk.STOCK_HELP, _('_Contents'), 'F1', None, self.show_help),
+            ('About', Gtk.STOCK_ABOUT, None, None, None, self.show_about),
             ])
         self.main_actions.add_toggle_actions([
             ('AlwaysShowPossible',
@@ -273,7 +252,7 @@ class UI (gconf_wrapper.GConfWrapper):
              _('Mark new additions in a separate color so you can keep track of them.'),
              self.tracker_toggle_cb, False),
             ('ToggleToolbar', None, _('Show _Toolbar'), None, None, self.toggle_toolbar_cb, True),
-            ('ToggleHighlight', gtk.STOCK_SELECT_COLOR, _('_Highlighter'),
+            ('ToggleHighlight', Gtk.STOCK_SELECT_COLOR, _('_Highlighter'),
              None, _('Highlight the current row, column and box'), self.toggle_highlight_cb, False)
             ])
 
@@ -312,28 +291,29 @@ class UI (gconf_wrapper.GConfWrapper):
             self.gsd.set_bg_color(bgcol)
 
     def setup_autosave (self):
-        gobject.timeout_add_seconds(self.gconf['auto_save_interval'] or 60, # in seconds...
+        GObject.timeout_add_seconds(self.gconf['auto_save_interval'] or 60, # in seconds...
                             self.autosave)
 
     def setup_main_boxes (self):
-        self.vb = gtk.VBox()
+        self.vb = Gtk.VBox()
         # Add menu bar and toolbar...
         mb = self.uimanager.get_widget('/MenuBar')
         mb.show()
-        self.vb.pack_start(mb, fill = False, expand = False)
+        self.vb.pack_start(mb, False, False, 0)
         self.tb = self.uimanager.get_widget('/Toolbar')
-        self.vb.pack_start(self.tb, fill = False, expand = False)
-        self.main_area = gtk.HBox()
+        self.tb.get_style_context().add_class(Gtk.STYLE_CLASS_PRIMARY_TOOLBAR)
+        self.vb.pack_start(self.tb, False, False, 0)
+        self.main_area = Gtk.HBox()
         self.swallower = dialog_swallower.SwappableArea(self.main_area)
         self.swallower.show()
-        self.vb.pack_start(self.swallower, True, padding = 12)
-        self.main_area.pack_start(self.gsd, padding = 6)
+        self.vb.pack_start(self.swallower, True, True, 12)
+        self.main_area.pack_start(self.gsd, True, True, 6)
         self.main_actions.set_visible(True)
-        self.game_box = gtk.VBox()
+        self.game_box = Gtk.VBox()
         self.main_area.show()
         self.vb.show()
         self.game_box.show()
-        self.main_area.pack_start(self.game_box, False, padding = 12)
+        self.main_area.pack_start(self.game_box, False, False, 12)
         self.w.add(self.vb)
 
     def setup_toggles (self):
@@ -419,7 +399,7 @@ class UI (gconf_wrapper.GConfWrapper):
                                  self.gsd.impossible_hints) % {'n':self.gsd.impossible_hints}
             sublabel += "\n"
         self.start_dancer()
-        dialog_extras.show_message(_("You win!"), label = _("You win!"),
+        dialog_extras.show_message_dialog(_("You win!"), label = _("You win!"),
                                    sublabel = sublabel
                                    )
 
@@ -436,7 +416,7 @@ class UI (gconf_wrapper.GConfWrapper):
     def new_cb (self, *args):
         if (self.gsd.grid and self.gsd.grid.is_changed() and not self.won):
             try:
-                if dialog_extras.getBoolean(
+                if dialog_extras.show_boolean_dialog(
                     label = _("Save this game before starting new one?"),
                     custom_yes = _("_Save game for later"),
                     custom_no = _("_Abandon game"),
@@ -457,7 +437,7 @@ class UI (gconf_wrapper.GConfWrapper):
             and self.gsd.grid.is_changed()
             and (not self.won)):
             try:
-                if dialog_extras.getBoolean(label = _("Save game before closing?")):
+                if dialog_extras.show_boolean_dialog(label = _("Save game before closing?")):
                     self.save_game(self)
             except dialog_extras.UserCancelledError:
                 return
@@ -472,7 +452,7 @@ class UI (gconf_wrapper.GConfWrapper):
         self.old_tracker_view = None
 
     @simple_debug
-    def resize_cb (self, widget, event):
+    def resize_cb (self, widget, event, user_data=None):
         self.gconf['width'] = event.width
         self.gconf['height'] = event.height
 
@@ -483,27 +463,27 @@ class UI (gconf_wrapper.GConfWrapper):
             and self.gsd.grid.is_changed()
             and (not self.won)):
             self.save_game(self)
-        if gtk.main_level() > 1:
+        if Gtk.main_level() > 1:
             # If we are in an embedded mainloop, that means that one
             # of our "swallowed" dialogs is active, in which case we
             # have to quit that mainloop before we can quit
             # properly.
             if self.swallower.running:
                 d = self.swallower.running
-                d.response(gtk.RESPONSE_DELETE_EVENT)
-            gtk.main_quit() # Quit the embedded mainloop
-            gobject.idle_add(self.quit_cb, 100) # Call ourselves again
+                d.response(Gtk.ResponseType.DELETE_EVENT)
+            Gtk.main_quit() # Quit the embedded mainloop
+            GObject.idle_add(self.quit_cb, 100) # Call ourselves again
                                                # to quit the main
                                                # mainloop
             return
         # make sure we really go away before doing our saving --
         # otherwise we appear sluggish.
-        while gtk.events_pending():
-            gtk.main_iteration()
+        while Gtk.events_pending():
+            Gtk.main_iteration()
         self.stop_worker_thread()
         # allow KeyboardInterrupts, which calls quit_cb outside the main loop
         try:
-            gtk.main_quit()
+            Gtk.main_quit()
         except RuntimeError:
             pass
 
@@ -601,7 +581,7 @@ class UI (gconf_wrapper.GConfWrapper):
         self.gsd.show_hint()
 
     @simple_debug
-    def auto_hint_cb (self, action):
+    def auto_hint_cb (self, action, user_data=None):
         if action.get_active():
             self.gsd.always_show_hints = True
             self.gsd.update_all_hints()
@@ -610,7 +590,7 @@ class UI (gconf_wrapper.GConfWrapper):
             self.gsd.clear_notes('AutoHint')
 
     @simple_debug
-    def impossible_implication_cb (self, action):
+    def impossible_implication_cb (self, action, user_data=None):
         if action.get_active():
             self.gsd.display_impossible_implications()
         else:
@@ -626,7 +606,7 @@ class UI (gconf_wrapper.GConfWrapper):
         self.game_box.add(self.tracker_ui)
 
     @simple_debug
-    def tracker_toggle_cb (self, widg):
+    def tracker_toggle_cb (self, widg, user_data=None):
         if widg.get_active():
             if self.old_tracker_view:
                 self.tinfo.set_tracker_view(self.old_tracker_view)
@@ -639,13 +619,13 @@ class UI (gconf_wrapper.GConfWrapper):
             self.tracker_ui.hide()
 
     @simple_debug
-    def toggle_toolbar_cb (self, widg):
+    def toggle_toolbar_cb (self, widg, user_data=None):
         if widg.get_active():
             self.tb.show()
         else:
             self.tb.hide()
 
-    def toggle_highlight_cb (self, widg):
+    def toggle_highlight_cb (self, widg, user_data=None):
         if widg.get_active():
             self.gsd.toggle_highlight(True)
         else:
@@ -654,7 +634,7 @@ class UI (gconf_wrapper.GConfWrapper):
     @simple_debug
     def show_info_cb (self, *args):
         if not self.gsd.grid:
-            dialog_extras.show_message(parent = self.w,
+            dialog_extras.show_message_dialog(parent = self.w,
                                        title = _("Puzzle Information"),
                                        label = _("There is no current puzzle.")
                                        )
@@ -679,7 +659,7 @@ class UI (gconf_wrapper.GConfWrapper):
         information += "\n"
         information += _("Amount of trial-and-error required to solve: ")
         information += str(len(diff.guesses))
-        dialog_extras.show_message(parent = self.w,
+        dialog_extras.show_message_dialog(parent = self.w,
                                    title = _("Puzzle Statistics"),
                                    label = _("Puzzle Statistics"),
                                    sublabel = information)
@@ -694,7 +674,7 @@ class UI (gconf_wrapper.GConfWrapper):
 
     @simple_debug
     def show_about (self, *args):
-        about = gtk.AboutDialog()
+        about = Gtk.AboutDialog()
         about.set_transient_for(self.w)
         about.set_name(APPNAME)
         about.set_version(VERSION)
@@ -713,8 +693,8 @@ class UI (gconf_wrapper.GConfWrapper):
     @simple_debug
     def show_help (self, *args):
         try:
-            gtk.show_uri(self.w.get_screen(), "ghelp:gnome-sudoku", gtk.get_current_event_time())
-        except gobject.GError, error:
+            Gtk.show_uri(self.w.get_screen(), "ghelp:gnome-sudoku", Gtk.get_current_event_time())
+        except GObject.GError, error:
             # FIXME: This should create a pop-up dialog
             print _('Unable to display help: %s') % str(error)
 
@@ -727,13 +707,14 @@ class UI (gconf_wrapper.GConfWrapper):
         gp = printing.GamePrinter(self.sudoku_maker, self.gconf)
         gp.run_dialog()
 
-class TrackerBox (gtk.VBox):
+class TrackerBox (Gtk.VBox):
 
     @simple_debug
     def __init__ (self, main_ui):
-
-        gtk.VBox.__init__(self)
-        self.builder = gtk.Builder()
+        GObject.GObject.__init__(self)
+        self.footprint_img =  cairo.ImageSurface.create_from_png(
+                                    os.path.join(IMAGE_DIR, "footprints.png"));
+        self.builder = Gtk.Builder()
         self.builder.set_translation_domain(DOMAIN)
         self.builder.add_from_file(os.path.join(UI_DIR, 'tracker.ui'))
         self.main_ui = main_ui
@@ -741,7 +722,7 @@ class TrackerBox (gtk.VBox):
         self.tinfo.ui = self
         self.vb = self.builder.get_object('vbox1')
         self.vb.unparent()
-        self.pack_start(self.vb, expand = True, fill = True)
+        self.pack_start(self.vb, True, True, 0)
         self.setup_actions()
         self.setup_tree()
         self.show_all()
@@ -758,18 +739,17 @@ class TrackerBox (gtk.VBox):
     @simple_debug
     def setup_tree (self):
         self.tracker_tree = self.builder.get_object('TrackerTreeView')
-        self.tracker_model = gtk.ListStore(int, gtk.gdk.Pixbuf, str)
-        self.tracker_model.set_sort_column_id(0, gtk.SORT_ASCENDING)
+        self.tracker_model = Gtk.ListStore(int, GdkPixbuf.Pixbuf, str)
+        self.tracker_model.set_sort_column_id(0, Gtk.SortType.ASCENDING)
         self.tracker_tree.set_model(self.tracker_model)
-        col1 = gtk.TreeViewColumn("", gtk.CellRendererPixbuf(), pixbuf = 1)
-        rend = gtk.CellRendererText()
-        col2 = gtk.TreeViewColumn("", rend, text = 2)
+        col1 = Gtk.TreeViewColumn("", Gtk.CellRendererPixbuf(), pixbuf = 1)
+        rend = Gtk.CellRendererText()
+        col2 = Gtk.TreeViewColumn("", rend, text = 2)
         col2.set_cell_data_func(rend, self.draw_tracker_name)
         self.tracker_tree.append_column(col2)
         self.tracker_tree.append_column(col1)
         # Our initial row...
-        pixbuf = self.pixbuf_transform_color(
-            STOCK_PIXBUFS['tracks'],
+        pixbuf = self.get_tracker_pixbuf(
             (0, 0, 0)
             )
         self.tracker_model.append([-1, pixbuf, _('Untracked')])
@@ -777,44 +757,47 @@ class TrackerBox (gtk.VBox):
 
     @simple_debug
     def setup_actions (self):
-        self.tracker_actions = gtk.ActionGroup('tracker_actions')
+        self.tracker_actions = Gtk.ActionGroup('tracker_actions')
         self.tracker_actions.add_actions(
             [('Remove',
-              gtk.STOCK_CLEAR,
+              Gtk.STOCK_CLEAR,
               _('_Remove'),
               None, _('Delete selected tracker.'),
               self.remove_tracker_cb
               ),
              ('Hide',
-              gtk.STOCK_CLEAR,
+              Gtk.STOCK_CLEAR,
               _('H_ide'),
               None, _('Hide current tracker entries.'),
               self.hide_tracker_cb
               ),
              ('Apply',
-              gtk.STOCK_APPLY,
+              Gtk.STOCK_APPLY,
               _('A_pply'),
               None, _('Apply all tracked values and remove the tracker.'),
               self.apply_tracker_cb
               ),
              ]
             )
-        a = self.tracker_actions.get_action('Remove')
-        a.connect_proxy(self.builder.get_object('RemoveTrackerButton'))
-        a = self.tracker_actions.get_action('Hide')
-        a.connect_proxy(self.builder.get_object('HideTrackerButton'))
-        a = self.tracker_actions.get_action('Apply')
-        a.connect_proxy(self.builder.get_object('ApplyTrackerButton'))
+        o = self.builder.get_object('RemoveTrackerButton')
+        o.set_related_action(self.tracker_actions.get_action('Remove'))
+
+        o = self.builder.get_object('HideTrackerButton')
+        o.set_related_action(self.tracker_actions.get_action('Hide'))
+
+        o = self.builder.get_object('ApplyTrackerButton')
+        o.set_related_action(self.tracker_actions.get_action('Apply'))
+
         self.builder.get_object('AddTrackerButton').connect('clicked',
                                                           self.add_tracker)
         # Default to insensitive (they only become sensitive once a tracker is added)
         self.tracker_actions.set_sensitive(False)
 
-    def draw_tracker_name(self, column, cell, model, iter):
+    def draw_tracker_name(self, column, cell, model, iter, user_data=None):
         if model.get_value(iter, 0) == self.tinfo.showing_tracker:
-            cell.set_property('weight', pango.WEIGHT_BOLD)
+            cell.set_property('weight', Pango.Weight.BOLD)
         else:
-            cell.set_property('weight', pango.WEIGHT_NORMAL)
+            cell.set_property('weight', Pango.Weight.NORMAL)
 
     @simple_debug
     def add_tracker (self, *args, **keys):
@@ -822,8 +805,7 @@ class TrackerBox (gtk.VBox):
             tracker_id = self.tinfo.create_tracker(keys['tracker_id'])
         else:
             tracker_id = self.tinfo.create_tracker()
-        pixbuf = self.pixbuf_transform_color(
-            STOCK_PIXBUFS['tracks'],
+        pixbuf = self.get_tracker_pixbuf(
             self.tinfo.get_color(tracker_id)
             )
         # select our new tracker
@@ -836,19 +818,14 @@ class TrackerBox (gtk.VBox):
         self.tinfo.set_tracker(tracker_id)
 
     @simple_debug
-    def pixbuf_transform_color (self, pixbuf, color):
-        """Return new pixbuf with color changed to color"""
-        pixbuf_str = pixbuf.get_pixels()
-        pixbuf_str_new = ""
+    def get_tracker_pixbuf (self, color):
+        surface = cairo.ImageSurface (cairo.FORMAT_ARGB32, 64, 64)
+        cr = cairo.Context(surface)
+        cr.set_source_rgb(*color);
+        cr.mask_surface(self.footprint_img, 0, 0)
+        cr.fill()
 
-        for alpha in pixbuf_str[3::4]:
-            pixbuf_str_new += chr(int(color[0]*255))
-            pixbuf_str_new += chr(int(color[1]*255))
-            pixbuf_str_new += chr(int(color[2]*255))
-            pixbuf_str_new += alpha
-
-        return gtk.gdk.pixbuf_new_from_data(pixbuf_str_new, gtk.gdk.COLORSPACE_RGB, True, 8, 
-                                            pixbuf.get_width(), pixbuf.get_height(), pixbuf.get_rowstride())
+        return Gdk.pixbuf_get_from_surface(surface, 0, 0, 64, 64)
 
     @simple_debug
     def find_tracker (self, tracker_id):
@@ -875,7 +852,7 @@ class TrackerBox (gtk.VBox):
             action.set_sensitive(self.tinfo.showing_tracker != tracker_info.NO_TRACKER)
 
     @simple_debug
-    def selection_changed_cb (self, selection):
+    def selection_changed_cb (self, selection, user_data=None):
         mod, itr = selection.get_selected()
         if itr:
             selected_tracker_id = mod.get_value(itr, 0)
@@ -895,7 +872,7 @@ class TrackerBox (gtk.VBox):
             self.main_ui.gsd.update_all_hints()
 
     @simple_debug
-    def remove_tracker_cb (self, action):
+    def remove_tracker_cb (self, action, user_data=None):
         mod, itr = self.tracker_tree.get_selection().get_selected()
         # This should only be called if there is an itr, but we'll
         # double-check just in case.
@@ -908,7 +885,7 @@ class TrackerBox (gtk.VBox):
             clearer.perform()
 
     @simple_debug
-    def hide_tracker_cb (self, action):
+    def hide_tracker_cb (self, action, user_data=None):
         hiding_tracker = self.tinfo.showing_tracker
         self.select_tracker(tracker_info.NO_TRACKER)
         self.main_ui.gsd.cover_track(True)
@@ -918,7 +895,7 @@ class TrackerBox (gtk.VBox):
         self.redraw_row(tracker_info.NO_TRACKER)
 
     @simple_debug
-    def apply_tracker_cb (self, action):
+    def apply_tracker_cb (self, action, user_data=None):
         '''Apply Tracker button action
         '''
         # Shouldn't be here if no tracker is showing
@@ -1011,7 +988,7 @@ def start_game ():
 
     ##  You must call g_thread_init() before executing any other GLib
     ##  functions in a threaded GLib program.
-    gobject.threads_init()
+    GObject.threads_init()
 
     if options.profile:
         options.profile = False
@@ -1021,7 +998,7 @@ def start_game ():
     u = UI()
     if not u.quit:
         try:
-            gtk.main()
+            Gtk.main()
         except KeyboardInterrupt:
             # properly quit on a keyboard interrupt...
             u.quit_cb()
