@@ -1,46 +1,60 @@
 # -*- coding: utf-8 -*-
 #!/usr/bin/python
 
-import gtk, gobject, pango, cairo
+from gi.repository import Gtk,Gdk,GObject,Pango,PangoCairo
 import math
+import random
 import tracker_info
 from gettext import gettext as _
 
+from simple_debug import simple_debug
+
 ERROR_HIGHLIGHT_COLOR = (1.0, 0, 0)
 
 BASE_SIZE = 35 # The "normal" size of a box (in pixels)
 
 # And the standard font-sizes -- these should fit nicely with the
 # BASE_SIZE
-BASE_FONT_SIZE = pango.SCALE * 13
-NOTE_FONT_SIZE = pango.SCALE * 6
+BASE_FONT_SIZE = Pango.SCALE * 13
+NOTE_FONT_SIZE = Pango.SCALE * 6
 
 BORDER_WIDTH = 9.0 # The size of space we leave for a box
 NORMAL_LINE_WIDTH = 1 # The size of the line we draw around a box
 
-class NumberSelector (gtk.EventBox):
+DEBUG_COLORS = True
+
+def debug_set_color_rgba(cr, rgba):
+    COLORS = ("red","green","blue","yellow","purple","wheat","maroon","gray")
+
+    if DEBUG_COLORS:
+        rgba = Gdk.RGBA()
+        rgba.parse(COLORS[random.randint(0,len(COLORS)-1)])
+
+    Gdk.cairo_set_source_rgba(cr, rgba)
+
+class NumberSelector (Gtk.EventBox):
 
     __gsignals__ = {
-        'changed':(gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
+        'changed':(GObject.SignalFlags.RUN_LAST, None, ()),
         }
 
     def __init__ (self, default = None, upper = 9):
         self.value = default
-        gtk.EventBox.__init__(self)
-        self.table = gtk.Table()
+        GObject.GObject.__init__(self)
+        self.table = Gtk.Table()
         self.add(self.table)
         side = int(math.sqrt(upper))
         n = 1
         for y in range(side):
             for x in range(side):
-                b = gtk.Button()
-                l = gtk.Label()
+                b = Gtk.Button()
+                l = Gtk.Label()
                 if n == self.value:
                     l.set_markup('<b><span size="x-small">%s</span></b>'%n)
                 else:
                     l.set_markup('<span size="x-small">%s</span>'%n)
                 b.add(l)
-                b.set_relief(gtk.RELIEF_HALF)
+                b.set_relief(Gtk.ReliefStyle.HALF)
                 l = b.get_children()[0]
                 b.set_border_width(0)
                 l.set_padding(0, 0)
@@ -49,8 +63,8 @@ class NumberSelector (gtk.EventBox):
                 self.table.attach(b, x, x+1, y, y+1)
                 n += 1
         if self.value:
-            db = gtk.Button()
-            l = gtk.Label()
+            db = Gtk.Button()
+            l = Gtk.Label()
             l.set_markup_with_mnemonic('<span size="x-small">'+_('_Clear')+'</span>')
             db.add(l)
             l.show()
@@ -68,7 +82,7 @@ class NumberSelector (gtk.EventBox):
     def set_value (self, n):
         self.value = n
 
-class NumberBox (gtk.Widget):
+class NumberBox (Gtk.DrawingArea):
 
     text = ''
     top_note_text = ''
@@ -84,38 +98,39 @@ class NumberBox (gtk.Widget):
     border_color = None
 
     __gsignals__ = {
-        'value-about-to-change':(gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
-        'notes-about-to-change':(gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
-        'changed':(gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
+        'value-about-to-change':(GObject.SignalFlags.RUN_LAST, None, ()),
+        'notes-about-to-change':(GObject.SignalFlags.RUN_LAST, None, ()),
+        'changed':(GObject.SignalFlags.RUN_LAST, None, ()),
         # undo-change - A hacky way to handle the fact that we want to
         # respond to undo's changes but we don't want undo to respond
         # to itself...
-        'undo-change':(gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
-        'notes-changed':(gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
+        'undo-change':(GObject.SignalFlags.RUN_LAST, None, ()),
+        'notes-changed':(GObject.SignalFlags.RUN_LAST, None, ()),
         }
 
-    base_state = gtk.STATE_NORMAL
+    base_state = Gtk.StateFlags.NORMAL
     npicker = None
     draw_boxes = False
 
     def __init__ (self, upper = 9, text = ''):
-        gtk.Widget.__init__(self)
+        Gtk.DrawingArea.__init__(self)
         self.upper = upper
         self.parent_win = None
         self.timer = None
-        self.font = self.style.font_desc
+        self.font = self.get_style().font_desc
         self.font.set_size(BASE_FONT_SIZE)
         self.note_font = self.font.copy()
         self.note_font.set_size(NOTE_FONT_SIZE)
-        self._top_note_layout = pango.Layout(self.create_pango_context())
+        self._top_note_layout = Pango.Layout(self.create_pango_context())
         self._top_note_layout.set_font_description(self.note_font)
-        self._bottom_note_layout = pango.Layout(self.create_pango_context())
+        self._bottom_note_layout = Pango.Layout(self.create_pango_context())
         self._bottom_note_layout.set_font_description(self.note_font)
+        self._base_stateflags = Gtk.StateFlags.NORMAL
         self.top_note_list = []
         self.bottom_note_list = []
         self.tinfo = tracker_info.TrackerInfo()
         self.set_property('can-focus', True)
-        self.set_property('events', gtk.gdk.ALL_EVENTS_MASK)
+        self.set_property('events', Gdk.EventMask.ALL_EVENTS_MASK)
         self.connect('button-press-event', self.button_press_cb)
         self.connect('key-release-event', self.key_press_cb)
         self.connect('enter-notify-event', self.pointer_enter_cb)
@@ -133,19 +148,19 @@ class NumberBox (gtk.Widget):
 
     def pointer_enter_cb (self, *args):
         if not self.is_focus():
-            self.set_state(gtk.STATE_PRELIGHT)
+            self.set_state_flags(Gtk.StateFlags.PRELIGHT, False)
 
     def pointer_leave_cb (self, *args):
-        self.set_state(self.base_state)
+        self.set_state_flags(self._base_stateflags, True)
         self._toggle_box_drawing_(False)
 
     def focus_in_cb (self, *args):
-        self.set_state(gtk.STATE_SELECTED)
-        self.base_state = gtk.STATE_SELECTED
+        self.set_state_flags(Gtk.StateFlags.SELECTED, True)
+        self._base_stateflags = Gtk.StateFlags.SELECTED
 
     def focus_out_cb (self, *args):
-        self.set_state(gtk.STATE_NORMAL)
-        self.base_state = gtk.STATE_NORMAL
+        self.set_state_flags(Gtk.StateFlags.NORMAL, True)
+        self._base_stateflags = Gtk.StateFlags.NORMAL
         self.destroy_npicker()
 
     def destroy_npicker (self):
@@ -170,7 +185,7 @@ class NumberBox (gtk.Widget):
     def button_press_cb (self, w, e):
         if self.read_only:
             return
-        if e.type == gtk.gdk._2BUTTON_PRESS:
+        if e.type == Gdk.EventType._2BUTTON_PRESS:
             # ignore second click (this makes a double click in the
             # middle of a cell get us a display of the numbers, rather
             # than selecting a number.
@@ -198,7 +213,7 @@ class NumberBox (gtk.Widget):
             return
         if self.npicker: # kill number picker no matter what is pressed
             self.destroy_npicker()
-        txt = gtk.gdk.keyval_name(e.keyval)
+        txt = Gdk.keyval_name(e.keyval)
         if type(txt) == type(None):
             # Make sure we don't trigger on unplugging the A/C charger etc
             return
@@ -206,9 +221,9 @@ class NumberBox (gtk.Widget):
 
         # Add the new value if need be
         if txt in [str(n) for n in range(1, self.upper+1)]:
-            if e.state & gtk.gdk.CONTROL_MASK:
+            if e.state & Gdk.ModifierType.CONTROL_MASK:
                 self.add_note_text(txt, top = True)
-            elif e.state & gtk.gdk.MOD1_MASK:
+            elif e.state & Gdk.ModifierType.MOD1_MASK:
                 self.remove_note_text(txt, top = True)
             elif self.get_text() != txt or \
                 (self.tracker_id != tracker_info.NO_TRACKER and
@@ -220,12 +235,12 @@ class NumberBox (gtk.Widget):
         elif txt in ['0', 'Delete', 'BackSpace']:
             self.set_text_interactive('')
         elif txt in ['n', 'N']:
-            if e.state & gtk.gdk.MOD1_MASK:
+            if e.state & Gdk.ModifierType.MOD1_MASK:
                 self.set_note_text_interactive(top_text = '')
             else:
                 self.show_note_editor(top = True)
         elif txt in ['m', 'M']:
-            if e.state & gtk.gdk.MOD1_MASK:
+            if e.state & Gdk.ModifierType.MOD1_MASK:
                 self.set_note_text_interactive(bottom_text = '')
             else:
                 self.show_note_editor(top = False)
@@ -273,16 +288,16 @@ class NumberBox (gtk.Widget):
 
     def show_note_editor (self, top = True):
         alloc = self.get_allocation()
-        w = gtk.Window()
+        w = Gtk.Window()
         w.set_property('skip-pager-hint', True)
         w.set_property('skip-taskbar-hint', True)
         w.set_decorated(False)
-        w.set_position(gtk.WIN_POS_MOUSE)
+        w.set_position(Gtk.WindowPosition.MOUSE)
         w.set_size_request(alloc.width, alloc.height/2)
         if self.parent_win:
             w.set_transient_for(self.parent_win)
-        f = gtk.Frame()
-        e = gtk.Entry()
+        f = Gtk.Frame()
+        e = Gtk.Entry()
         f.add(e)
         if top:
             e.set_text(self.top_note_text)
@@ -294,7 +309,7 @@ class NumberBox (gtk.Widget):
         e.connect('focus-out-event', lambda e, ev, w: w.destroy(), w)
         e.connect('focus-out-event', self.note_focus_out)
         e.connect('activate', lambda e, w: w.destroy(), w)
-        x, y = self.window.get_origin()
+        _, x, y = self.get_window().get_origin()
         if top:
             w.move(x, y)
         else:
@@ -311,18 +326,17 @@ class NumberBox (gtk.Widget):
             self.set_text_interactive('')            
 
     def show_number_picker (self):
-        w = gtk.Window(type = gtk.WINDOW_POPUP)
+        w = Gtk.Window(type = Gtk.WindowType.POPUP)
         ns = NumberSelector(upper = self.upper, default = self.get_value())
         ns.connect('changed', self.number_changed_cb)
         w.grab_focus()
         w.add(ns)
-        r = w.get_allocation()
-        my_origin = self.window.get_origin()
-        x, y = self.window.get_size()
+        _, xorigin, yorigin = self.get_window().get_origin()
+        x, y = (self.get_allocated_width(), self.get_allocated_height())
         popupx, popupy = w.get_size()
         overlapx = popupx-x
         overlapy = popupy-y
-        w.move(my_origin[0]-(overlapx/2), my_origin[1]-(overlapy/2))
+        w.move(xorigin - (overlapx/2), yorigin - (overlapy/2))
         w.show()
         self.npicker = w
 
@@ -334,7 +348,7 @@ class NumberBox (gtk.Widget):
 
     def set_font (self, font):
         if type(font) == str:
-            font = pango.FontDescription(font)
+            font = Pango.FontDescription(font)
         self.font = font
         if self.text:
             self.set_text(self.text)
@@ -342,7 +356,7 @@ class NumberBox (gtk.Widget):
 
     def set_note_font (self, font):
         if type(font) == str:
-            font = pango.FontDescription(font)
+            font = Pango.FontDescription(font)
         self.note_font = font
         self._top_note_layout.set_font_description(font)
         self._bottom_note_layout.set_font_description(font)
@@ -357,9 +371,9 @@ class NumberBox (gtk.Widget):
         '''Display the notes for the current view
         '''
         self.top_note_text = self.get_note_display(self.top_note_list)[1]
-        self._top_note_layout.set_markup(self.get_note_display(self.top_note_list)[2])
+        self._top_note_layout.set_markup(self.get_note_display(self.top_note_list)[2], -1)
         self.bottom_note_text = self.get_note_display(self.bottom_note_list)[1]
-        self._bottom_note_layout.set_markup(self.get_note_display(self.bottom_note_list)[2])
+        self._bottom_note_layout.set_markup(self.get_note_display(self.bottom_note_list)[2], -1)
         self.queue_draw()
 
     def set_note_text (self, top_text = None, bottom_text = None, for_hint = False):
@@ -506,96 +520,34 @@ class NumberBox (gtk.Widget):
         self.top_note_list, self.bottom_note_list = notelists
         self.show_note_text()
 
-    def do_realize (self):
-        # The do_realize method is responsible for creating GDK (windowing system)
-        # resources. In this example we will create a new gdk.Window which we
-        # then draw on
+    @simple_debug
+    def do_draw(self, cr):
 
-        # First set an internal flag telling that we're realized
-        self.set_flags(self.flags() | gtk.REALIZED)
+        w = self.get_allocated_width()
+        h = self.get_allocated_height()
+        style_ctx = self.get_style_context()
 
-        # Create a new gdk.Window which we can draw on.
-        # Also say that we want to receive exposure events by setting
-        # the event_mask
-        self.window = gtk.gdk.Window(
-            self.get_parent_window(),
-            width = self.allocation.width,
-            height = self.allocation.height,
-            window_type = gtk.gdk.WINDOW_CHILD,
-            wclass = gtk.gdk.INPUT_OUTPUT,
-            event_mask = self.get_events() | gtk.gdk.EXPOSURE_MASK)
-
-        # Associate the gdk.Window with ourselves, Gtk+ needs a reference
-        # between the widget and the gdk window
-        self.window.set_user_data(self)
-
-        # Attach the style to the gdk.Window, a style contains colors and
-        # GC contextes used for drawing
-        self.style.attach(self.window)
-
-        # The default color of the background should be what
-        # the style (theme engine) tells us.
-        self.style.set_background(self.window, gtk.STATE_NORMAL)
-        self.window.move_resize(*self.allocation)
-
-    def do_unrealize (self):
-        # The do_unrealized method is responsible for freeing the GDK resources
-
-        # De-associate the window we created in do_realize with ourselves
-        self.window.set_user_data(None)
-
-    def do_size_request (self, requisition):
-        # The do_size_request method Gtk+ is calling on a widget to ask
-        # it the widget how large it wishes to be. It's not guaranteed
-        # that gtk+ will actually give this size to the widget
-
-        # In this case, we say that we want to be as big as the
-        # text is, and a square
-        width, height = self._layout.get_size()
-        if width > height:
-            side = width/pango.SCALE
-        else:
-            side = height/pango.SCALE
-        (requisition.width, requisition.height) = (side, side)
-
-    def do_size_allocate(self, allocation):
-        # The do_size_allocate is called by when the actual size is known
-        # and the widget is told how much space could actually be allocated
-
-        # Save the allocated space
-        self.allocation = allocation
-
-        # If we're realized, move and resize the window to the
-        # requested coordinates/positions
-        if self.flags() & gtk.REALIZED:
-            self.window.move_resize(*allocation)
-
-    def do_expose_event(self, event):
-        # The do_expose_event is called when the widget is asked to draw itself
-        # Remember that this will be called a lot of times, so it's usually
-        # a good idea to write this code as optimized as it can be, don't
-        # Create any resources in here.
-        x, y, w, h = self.allocation
-        cr = self.window.cairo_create()
-        self.draw_background_color(cr, w, h)
+        self.draw_background_color(cr, style_ctx, w, h)
         if self.is_focus():
-            self.draw_highlight_box(cr, w, h)
+            self.draw_highlight_box(cr, style_ctx, w, h)
         if self.border_color is not None:
             border_width = 3.0
             cr.set_source_rgb(*self.border_color)
             cr.rectangle(border_width*0.5, border_width*0.5, w-border_width, h-border_width)
             cr.set_line_width(border_width)
             cr.stroke()
+
         if h < w:
             scale = h/float(BASE_SIZE)
         else:
             scale = w/float(BASE_SIZE)
         cr.scale(scale, scale)
-        self.draw_text(cr)
-        if self.draw_boxes and self.is_focus():
-            self.draw_note_area_highlight_box(cr)
 
-    def draw_background_color (self, cr, w, h):
+        self.draw_text(cr, style_ctx)
+        if self.draw_boxes and self.is_focus():
+            self.draw_note_area_highlight_box(cr, style_ctx)
+
+    def draw_background_color (self, cr, style_ctx, w, h):
         if self.read_only:
             if self.custom_background_color:
                 r, g, b = self.custom_background_color
@@ -603,24 +555,37 @@ class NumberBox (gtk.Widget):
                     r*0.6, g*0.6, b*0.6
                     )
             else:
-                cr.set_source_color(self.style.base[gtk.STATE_INSENSITIVE])
+                #cr.set_source_color(self.style.base[Gtk.StateFlags.INSENSITIVE])
+                #Gdk.cairo_set_source_rgba(
+                debug_set_color_rgba(
+                        cr, style_ctx.get_color(Gtk.StateFlags.INSENSITIVE))
         elif self.is_focus():
-            cr.set_source_color(self.style.base[gtk.STATE_SELECTED])
+            #cr.set_source_color(self.style.base[Gtk.StateFlags.SELECTED])
+            #Gdk.cairo_set_source_rgba(
+            debug_set_color_rgba(
+                    cr, style_ctx.get_color(Gtk.StateFlags.SELECTED))
         elif self.custom_background_color:
             cr.set_source_rgb(*self.custom_background_color)
         else:
-            cr.set_source_color(
-                self.style.base[self.state]
-                )
+            #cr.set_source_color(
+            #    self.style.base[self.state]
+            #    )
+            #Gdk.cairo_set_source_rgba(
+            debug_set_color_rgba(
+                    cr, style_ctx.get_color(self.get_state_flags()))
         cr.rectangle(
             0, 0, w, h,
             )
         cr.fill()
 
-    def draw_highlight_box (self, cr, w, h):
-        cr.set_source_color(
-            self.style.base[gtk.STATE_SELECTED]
-            )
+    def draw_highlight_box (self, cr, style_ctx, w, h):
+        #cr.set_source_color(
+        #    self.style.base[Gtk.StateFlags.SELECTED]
+        #    )
+        #Gdk.cairo_set_source_rgba(
+        debug_set_color_rgba(
+                    cr, style_ctx.get_color(Gtk.StateFlags.SELECTED))
+
         border = 4 * w / BASE_SIZE
         cr.rectangle(
             # left-top
@@ -633,11 +598,15 @@ class NumberBox (gtk.Widget):
         cr.set_line_width(border)
         cr.stroke()
 
-    def draw_note_area_highlight_box (self, cr):
+    def draw_note_area_highlight_box (self, cr, style_ctx):
         # set up our paint brush...
-        cr.set_source_color(
-            self.style.mid[self.state]
-            )
+        #cr.set_source_color(
+        #    self.style.mid[self.state]
+        #    )
+        #Gdk.cairo_set_source_rgba(
+        debug_set_color_rgba(
+                cr, style_ctx.get_color(self.get_state_flags()))
+
         cr.set_line_width(NORMAL_LINE_WIDTH)
         # top rectangle
         cr.rectangle(NORMAL_LINE_WIDTH*0.5,
@@ -653,7 +622,7 @@ class NumberBox (gtk.Widget):
                      )
         cr.stroke()
 
-    def draw_text (self, cr):
+    def draw_text (self, cr, style_ctx):
         fontw, fonth = self._layout.get_pixel_size()
         # Draw a shadow for tracked conflicts.  This is done to
         # differentiate between tracked and untracked conflicts.
@@ -661,22 +630,34 @@ class NumberBox (gtk.Widget):
             cr.set_source_rgb(*self.shadow_color)
             for xoff, yoff in [(1,1),(2,2)]:
                 cr.move_to((BASE_SIZE/2)-(fontw/2) + xoff, (BASE_SIZE/2) - (fonth/2) + yoff)
-                cr.show_layout(self._layout)
+                PangoCairo.show_layout(cr, self._layout)
         if self.text_color:
             cr.set_source_rgb(*self.text_color)
         elif self.read_only:
-            cr.set_source_color(self.style.text[gtk.STATE_NORMAL])
+            #cr.set_source_color(self.style.text[Gtk.StateFlags.NORMAL])
+            #Gdk.cairo_set_source_rgba(
+            debug_set_color_rgba(
+                cr, style_ctx.get_color(Gtk.StateFlags.NORMAL))
         else:
-            cr.set_source_color(self.style.text[self.state])
+            #cr.set_source_color(self.style.text[self.state])
+            #Gdk.cairo_set_source_rgba(
+            debug_set_color_rgba(
+                cr, style_ctx.get_color(Gtk.StateFlags.NORMAL))
+
         # And draw the text in the middle of the allocated space
         if self._layout:
             cr.move_to(
                 (BASE_SIZE/2)-(fontw/2),
                 (BASE_SIZE/2) - (fonth/2),
                 )
-            cr.update_layout(self._layout)
-            cr.show_layout(self._layout)
-        cr.set_source_color(self.style.text[self.state])
+            PangoCairo.update_layout(cr, self._layout)
+            PangoCairo.show_layout(cr, self._layout)
+
+        #cr.set_source_color(self.style.text[self.state])
+        #Gdk.cairo_set_source_rgba(
+        debug_set_color_rgba(
+            cr, style_ctx.get_color(Gtk.StateFlags.NORMAL))
+
         # And draw any note text...
         if self._top_note_layout:
             fontw, fonth = self._top_note_layout.get_pixel_size()
@@ -684,16 +665,16 @@ class NumberBox (gtk.Widget):
                 NORMAL_LINE_WIDTH,
                 0,
                 )
-            cr.update_layout(self._top_note_layout)
-            cr.show_layout(self._top_note_layout)
+            PangoCairo.update_layout(cr, self._top_note_layout)
+            PangoCairo.show_layout(cr, self._top_note_layout)
         if self._bottom_note_layout:
             fontw, fonth = self._bottom_note_layout.get_pixel_size()
             cr.move_to(
                 NORMAL_LINE_WIDTH,
                 BASE_SIZE-fonth,
                 )
-            cr.update_layout(self._bottom_note_layout)
-            cr.show_layout(self._bottom_note_layout)
+            PangoCairo.update_layout(cr, self._bottom_note_layout)
+            PangoCairo.show_layout(cr, self._bottom_note_layout)
 
     def set_text_color (self, color, shadow = None):
         self.shadow_color = shadow
@@ -779,7 +760,7 @@ class SudokuNumberBox (NumberBox):
         if not hasattr(self, 'bold_font'):
             self.normal_font = self.font
             self.bold_font = self.font.copy()
-            self.bold_font.set_weight(pango.WEIGHT_BOLD)
+            self.bold_font.set_weight(Pango.Weight.BOLD)
         if self.read_only:
             self.set_font(self.bold_font)
         else:
@@ -797,11 +778,11 @@ class SudokuNumberBox (NumberBox):
         self.queue_draw()
 
 
-gobject.type_register(NumberBox)
+GObject.type_register(NumberBox)
 
 if __name__ == '__main__':
-    window = gtk.Window()
-    window.connect('delete-event', gtk.main_quit)
+    window = Gtk.Window()
+    window.connect('delete-event', Gtk.main_quit)
 
     def test_number_selector ():
         nselector = NumberSelector(default = 3)
@@ -818,4 +799,4 @@ if __name__ == '__main__':
 #    test_number_selector()
     test_number_box()
     window.show_all()
-    gtk.main()
+    Gtk.main()
