@@ -475,7 +475,7 @@ about_callback (void)
 			 "license", license, "authors", authors, "artists",
 			 artists, "documenters", documenters,
 			 "translator-credits", _("translator-credits"),
-			 "logo-icon-name", "gnome-mines", "website",
+			 "logo-icon-name", "gnomine", "website",
 			 "http://www.gnome.org/projects/gnome-games/",
 			 "website-label", _("GNOME Games web site"),
 			 "wrap-license", TRUE, NULL);
