@@ -11,9 +11,18 @@
 #include <config.h>
 
 #include "games-clock.h"
+#include "games-glib-compat.h"
 
 G_DEFINE_TYPE (GamesClock, games_clock, GTK_TYPE_LABEL)
 
+struct GamesClockPrivate {
+  guint update_timeout_id;
+  gboolean update;
+  gboolean started;
+  time_t start_time;
+  time_t stop_time;
+};
+
 static void
 clock_paint (GamesClock *clock_widget)
 {
@@ -46,23 +55,19 @@ games_clock_update (GamesClock *clock_widget)
 static void
 games_clock_start_timer (GamesClock *clock_widget)
 {
-  if (clock_widget->update_timeout_id != 0)
+  if (clock_widget->priv->update_timeout_id != 0)
     return;
 
-  clock_widget->update_timeout_id =
-#if GLIB_CHECK_VERSION (2, 14, 0)
-    g_timeout_add_seconds (1, (GSourceFunc) games_clock_update, clock_widget);
-#else
-    g_timeout_add (1000, (GSourceFunc) games_clock_update, clock_widget);
-#endif
+  clock_widget->priv->update_timeout_id =
+    gdk_threads_add_timeout_seconds (1, (GSourceFunc) games_clock_update, clock_widget);
 }
 
 static void
 games_clock_stop_timer (GamesClock *clock_widget)
 {
-  if (clock_widget->update_timeout_id != 0) {
-    g_source_remove (clock_widget->update_timeout_id);
-    clock_widget->update_timeout_id = 0;
+  if (clock_widget->priv->update_timeout_id != 0) {
+    g_source_remove (clock_widget->priv->update_timeout_id);
+    clock_widget->priv->update_timeout_id = 0;
   }
 }
 
@@ -82,20 +87,31 @@ games_clock_class_init (GamesClockClass * klass)
   GObjectClass *object_class = (GObjectClass *) klass;
 
   object_class->finalize = games_clock_finalize;
+
+  g_type_class_add_private (object_class, sizeof (GamesClockPrivate));
 }
 
 static void
 games_clock_init (GamesClock *clock_widget)
 {
-  clock_widget->update_timeout_id = 0;
-  clock_widget->start_time = clock_widget->stop_time = 0;
-  clock_widget->started = FALSE;
-  clock_widget->update = TRUE;
+  clock_widget->priv = G_TYPE_INSTANCE_GET_PRIVATE (clock_widget, GAMES_TYPE_CLOCK, GamesClockPrivate);
+
+  clock_widget->priv->update_timeout_id = 0;
+  clock_widget->priv->start_time = clock_widget->priv->stop_time = 0;
+  clock_widget->priv->started = FALSE;
+  clock_widget->priv->update = TRUE;
 
   /* FIXMEchpe: call clock_paint() instead */
   gtk_label_set_text (GTK_LABEL (clock_widget), "00:00:00");
 }
 
+/**
+ * games_clock_new:
+ * 
+ * Create a new game clock
+ * 
+ * Returns: A new #GamesClock
+ **/
 GtkWidget *
 games_clock_new (void)
 {
@@ -115,13 +131,13 @@ games_clock_start (GamesClock *clock_widget)
 {
   g_return_if_fail (GAMES_IS_CLOCK (clock_widget));
 
-  if (clock_widget->started)
+  if (clock_widget->priv->started)
     return; /* nothing to do */
 
-  clock_widget->started = TRUE;
-  clock_widget->start_time = time (NULL) - (clock_widget->stop_time - clock_widget->start_time);
+  clock_widget->priv->started = TRUE;
+  clock_widget->priv->start_time = time (NULL) - (clock_widget->priv->stop_time - clock_widget->priv->start_time);
 
-  if (clock_widget->update)
+  if (clock_widget->priv->update)
     games_clock_start_timer (clock_widget);
 }
 
@@ -136,7 +152,7 @@ games_clock_is_started   (GamesClock *clock_widget)
 {
   g_return_val_if_fail (GAMES_IS_CLOCK (clock_widget), FALSE);
 
-  return clock_widget->started;
+  return clock_widget->priv->started;
 }
 
 /**
@@ -151,11 +167,11 @@ games_clock_stop (GamesClock *clock_widget)
 {
   g_return_if_fail (GAMES_IS_CLOCK (clock_widget));
 
-  if (!clock_widget->started)
+  if (!clock_widget->priv->started)
     return;
 
-  clock_widget->started = FALSE;
-  clock_widget->stop_time = time (NULL);
+  clock_widget->priv->started = FALSE;
+  clock_widget->priv->stop_time = time (NULL);
 
   games_clock_stop_timer (clock_widget);
   clock_paint (clock_widget);
@@ -172,7 +188,7 @@ games_clock_reset (GamesClock *clock_widget)
 {
   g_return_if_fail (GAMES_IS_CLOCK (clock_widget));
 
-  clock_widget->start_time = clock_widget->stop_time = time (NULL);
+  clock_widget->priv->start_time = clock_widget->priv->stop_time = time (NULL);
 
   clock_paint (clock_widget);
 }
@@ -188,10 +204,10 @@ games_clock_get_seconds (GamesClock *clock_widget)
 {
   g_return_val_if_fail (GAMES_IS_CLOCK (clock_widget), 0);
 
-  if (clock_widget->started)
-    return time (NULL) - clock_widget->start_time;
+  if (clock_widget->priv->started)
+    return time (NULL) - clock_widget->priv->start_time;
   else
-    return clock_widget->stop_time - clock_widget->start_time;
+    return clock_widget->priv->stop_time - clock_widget->priv->start_time;
 }
 
 /**
@@ -207,12 +223,12 @@ games_clock_add_seconds (GamesClock *clock_widget,
 {
   g_return_if_fail (GAMES_IS_CLOCK (clock_widget));
 
-  if (!clock_widget->started) {
+  if (!clock_widget->priv->started) {
     g_warning ("Clock not started, cannot add seconds!\n");
     return;
   }
 
-  clock_widget->start_time -= seconds;
+  clock_widget->priv->start_time -= seconds;
   clock_paint (clock_widget);
 }
 
@@ -232,10 +248,10 @@ games_clock_set_update (GamesClock *clock_widget,
   g_return_if_fail (GAMES_IS_CLOCK (clock_widget));
 
   do_update = do_update != FALSE;
-  if (do_update == clock_widget->update)
+  if (do_update == clock_widget->priv->update)
     return;
 
-  clock_widget->update = do_update;
+  clock_widget->priv->update = do_update;
   if (do_update) {
     games_clock_start_timer (clock_widget);
     clock_paint (clock_widget);
